@@ -2,6 +2,7 @@
     I rolled my own for following reasons: 
     1. I wanted something under MIT license (best alternative was licensed under GPL/BSD)
     2. I wanted singleton behavior so only a single exiftool process was ever running
+    3. When used as a context manager, I wanted the operations to batch until exiting the context (improved performance)
     If these aren't important to you, I highly recommend you use Sven Marnach's excellent 
     pyexiftool: https://github.com/smarnach/pyexiftool which provides more functionality """
 
@@ -10,10 +11,8 @@ import logging
 import os
 import shutil
 import subprocess
-import sys
 from functools import lru_cache  # pylint: disable=syntax-error
 
-from .utils import _debug
 
 # exiftool -stay_open commands outputs this EOF marker after command is run
 EXIFTOOL_STAYOPEN_EOF = "{ready}"
@@ -23,9 +22,7 @@ EXIFTOOL_STAYOPEN_EOF_LEN = len(EXIFTOOL_STAYOPEN_EOF)
 @lru_cache(maxsize=1)
 def get_exiftool_path():
     """ return path of exiftool, cache result """
-    exiftool_path = shutil.which('exiftool')
-    if _debug():
-        logging.debug("exiftool path = %s" % (exiftool_path))
+    exiftool_path = shutil.which("exiftool")
     if exiftool_path:
         return exiftool_path.rstrip()
     else:
@@ -59,7 +56,7 @@ class _ExifToolProc:
                 )
             return
 
-        self._exiftool = exiftool if exiftool else get_exiftool_path()
+        self._exiftool = exiftool or get_exiftool_path()
         self._process_running = False
         self._start_proc()
 
@@ -98,12 +95,12 @@ class _ExifToolProc:
                 "-",  # read from stdin
                 "-common_args",  # specifies args common to all commands subsequently run
                 "-n",  # no print conversion (e.g. print tag values in machine readable format)
-                "-P",  # Preserve file modification date/time (possible interfere w/ --touch-file)
+                "-P",  # Preserve file modification date/time
                 "-G",  # print group name for each tag
             ],
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
-            stderr=subprocess.DEVNULL,
+            stderr=subprocess.STDOUT,
         )
         self._process_running = True
 
@@ -136,38 +133,72 @@ class ExifTool:
     """ Basic exiftool interface for reading and writing EXIF tags """
 
     def __init__(self, filepath, exiftool=None, overwrite=True):
-        """ Return ExifTool object
-            file: path to image file
-            exiftool: path to exiftool, if not specified will look in path
-            overwrite: if True, will overwrite image file without creating backup, default=False """
+        """ Create ExifTool object
+
+        Args:
+                file: path to image file
+                exiftool: path to exiftool, if not specified will look in path
+                overwrite: if True, will overwrite image file without creating backup, default=False
+        Returns:
+            ExifTool instance
+        """
         self.file = filepath
         self.overwrite = overwrite
         self.data = {}
+        self.error = None
+        # if running as a context manager, self._context_mgr will be True
+        self._context_mgr = False
         self._exiftoolproc = _ExifToolProc(exiftool=exiftool)
         self._process = self._exiftoolproc.process
         self._read_exif()
 
     def setvalue(self, tag, value):
-        """ Set tag to value(s) 
-            if value is None, will delete tag """
+        """ Set tag to value(s); if value is None, will delete tag
+
+        Args:
+            tag: str; name of tag to set
+            value: str; value to set tag to
+        
+        Returns:
+            True if success otherwise False
+            
+            If error generated by exiftool, returns False and sets self.error to error string
+            If called in context manager, returns True (execution is delayed until exiting context manager)
+        """
 
         if value is None:
             value = ""
         command = [f"-{tag}={value}"]
-        if self.overwrite:
+        if self.overwrite and not self._context_mgr:
             command.append("-overwrite_original")
-        self.run_commands(*command)
+        if self._context_mgr:
+            self._commands.extend(command)
+            return True
+        else:
+            _, self.error = self.run_commands(*command)
+            return self.error is None
 
     def addvalues(self, tag, *values):
         """ Add one or more value(s) to tag
             If more than one value is passed, each value will be added to the tag
-            Notes: exiftool may add duplicate values for some tags so the caller must ensure
-                   the values being added are not already in the EXIF data
-                   For some tags, such as IPTC:Keywords, this will add a new value to the list of keywords, 
-                   but for others, such as EXIF:ISO, this will literally add a value to the existing value.
-                   It's up to the caller to know what exiftool will do for each tag
-                   If setvalue called before addvalues, exiftool does not appear to add duplicates, 
-                   but if addvalues called without first calling setvalue, exiftool will add duplicate values
+
+        Args:
+            tag: str; tag to set
+            *values: str; one or more values to set
+        
+        Returns:
+            True if success otherwise False
+            
+            If error generated by exiftool, returns False and sets self.error to error string
+            If called in context manager, returns True (execution is delayed until exiting context manager)
+            
+        Notes: exiftool may add duplicate values for some tags so the caller must ensure
+               the values being added are not already in the EXIF data
+               For some tags, such as IPTC:Keywords, this will add a new value to the list of keywords, 
+               but for others, such as EXIF:ISO, this will literally add a value to the existing value.
+               It's up to the caller to know what exiftool will do for each tag
+               If setvalue called before addvalues, exiftool does not appear to add duplicates, 
+               but if addvalues called without first calling setvalue, exiftool will add duplicate values
         """
         if not values:
             raise ValueError("Must pass at least one value")
@@ -178,23 +209,41 @@ class ExifTool:
                 raise ValueError("Can't add None value to tag")
             command.append(f"-{tag}+={value}")
 
-        if self.overwrite:
+        if self.overwrite and not self._context_mgr:
             command.append("-overwrite_original")
 
-        if command:
-            self.run_commands(*command)
+        if self._context_mgr:
+            self._commands.extend(command)
+            return True
+        else:
+            _, self.error = self.run_commands(*command)
+            return self.error is None
 
     def run_commands(self, *commands, no_file=False):
-        """ run commands in the exiftool process and return result 
-            no_file: (bool) do not pass the filename to exiftool (default=False)
-                     by default, all commands will be run against self.file
-                     use no_file=True to run a command without passing the filename """
+        """ Run commands in the exiftool process and return result.
+
+        Args:
+                *commands: exiftool commands to run
+                no_file: (bool) do not pass the filename to exiftool (default=False)
+                        by default, all commands will be run against self.file
+                        use no_file=True to run a command without passing the filename
+        Returns:
+            (output, errror)
+            output: bytes is containing output of exiftool commands
+            error: if exiftool generated an error, bytes containing error string otherwise None
+
+        Note: Also sets self.error if error generated.
+        """
         if not (hasattr(self, "_process") and self._process):
             raise ValueError("exiftool process is not running")
 
         if not commands:
             raise TypeError("must provide one or more command to run")
 
+        if self._context_mgr and self.overwrite:
+            commands = list(commands)
+            commands.append("-overwrite_original")
+
         filename = os.fsencode(self.file) if not no_file else b""
         command_str = (
             b"\n".join([c.encode("utf-8") for c in commands])
@@ -204,18 +253,22 @@ class ExifTool:
             + b"-execute\n"
         )
 
-        if _debug():
-            logging.debug(command_str)
-
         # send the command
         self._process.stdin.write(command_str)
         self._process.stdin.flush()
 
         # read the output
         output = b""
+        error = b""
         while EXIFTOOL_STAYOPEN_EOF not in str(output):
-            output += self._process.stdout.readline().strip()
-        return output[:-EXIFTOOL_STAYOPEN_EOF_LEN]
+            line = self._process.stdout.readline()
+            if line.startswith(b"Warning"):
+                error += line
+            else:
+                output += line.strip()
+        error = None if error == b"" else error
+        self.error = error
+        return output[:-EXIFTOOL_STAYOPEN_EOF_LEN], error
 
     @property
     def pid(self):
@@ -225,14 +278,14 @@ class ExifTool:
     @property
     def version(self):
         """ returns exiftool version """
-        ver = self.run_commands("-ver", no_file=True)
+        ver, _ = self.run_commands("-ver", no_file=True)
         return ver.decode("utf-8")
 
     def asdict(self):
         """ return dictionary of all EXIF tags and values from exiftool 
             returns empty dict if no tags
         """
-        json_str = self.run_commands("-json")
+        json_str, _ = self.run_commands("-json")
         if json_str:
             exifdict = json.loads(json_str)
             return exifdict[0]
@@ -241,7 +294,8 @@ class ExifTool:
 
     def json(self):
         """ returns JSON string containing all EXIF tags and values from exiftool """
-        return self.run_commands("-json")
+        json, _ = self.run_commands("-json")
+        return json
 
     def _read_exif(self):
         """ read exif data from file """
@@ -250,3 +304,14 @@ class ExifTool:
 
     def __str__(self):
         return f"file: {self.file}\nexiftool: {self._exiftoolproc._exiftool}"
+
+    def __enter__(self):
+        self._context_mgr = True
+        self._commands = []
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if exc_type:
+            return False
+        elif self._commands:
+            _, self.error = self.run_commands(*self._commands)
