@@ -91,9 +91,10 @@ class PhotoInfo:
             and self.raw_original
         ):
             # return the JPEG version as that's what Photos 5+ does
-            return self._info["raw_pair_info"]["originalFilename"]
+            original_name = self._info["raw_pair_info"]["originalFilename"]
         else:
-            return self._info["originalFilename"]
+            original_name = self._info["originalFilename"]
+        return original_name or self.filename
 
     @property
     def date(self):
@@ -164,6 +165,8 @@ class PhotoInfo:
                         photopath = os.path.join(
                             self._db._masters_path, self._info["imagePath"]
                         )
+                if not os.path.isfile(photopath):
+                    photopath = None
                 self._path = photopath
                 return photopath
 
@@ -175,6 +178,8 @@ class PhotoInfo:
                     self._info["directory"],
                     self._info["filename"],
                 )
+                if not os.path.isfile(photopath):
+                    photopath = None
                 self._path = photopath
                 return photopath
 
@@ -188,6 +193,8 @@ class PhotoInfo:
                     self._info["directory"],
                     self._info["filename"],
                 )
+            if not os.path.isfile(photopath):
+                photopath = None
             self._path = photopath
             return photopath
 
